/**
 * @file
 * JavaScript behaviors for the Revision Graph module.
 * 
 * This file provides the core functionality for fetching, processing and 
 * displaying revision history in a graphical format, similar to git commit graphs.
 * 
 * @example
 * // The module initializes automatically on pages with the revision graph element:
 * // <div id="js-revision-graph" data-total="25" data-limit="50" data-page="0"></div>
 */

(function (Drupal, once) {
  "use strict";

  /**
   * RevisionGraph namespace for module functionality.
   * @namespace
   */
  Drupal.revisionGraph = Drupal.revisionGraph || {};

  /**
   * @typedef {Object} RevisionGraphConfig
   * @property {number} fetchDelay - Milliseconds between batch fetches to prevent UI blocking
   * @property {Object} colorThresholds - Percentage thresholds for progress bar color changes
   * @property {number} colorThresholds.low - Upper threshold for low progress state (default: 30%)
   * @property {number} colorThresholds.medium - Upper threshold for medium progress state (default: 70%)
   */

  /**
   * Configuration options with defaults.
   * 
   * @type {RevisionGraphConfig}
   */
  Drupal.revisionGraph.config = {
    fetchDelay: 10,    // Delay between batch fetches (ms)
    colorThresholds: {  // Progress bar color thresholds
      low: 30,
      medium: 70
    }
  };

  /**
   * @typedef {Object} RevisionTag
   * @property {string} name - Identifier for the tag
   * @property {string} label - Human-readable label
   * @property {string} color - Hex color code for the tag
   */

  /**
   * @typedef {Object} RevisionUrl
   * @property {string} label - The text to display for the link
   * @property {string} url - The URL the link should navigate to
   */

  /**
   * @typedef {Object} RawRevisionData
   * @property {string} id - Unique identifier for the revision
   * @property {string} message - Revision message or description
   * @property {string} author - Name of the revision author
   * @property {number} timestamp - UNIX timestamp of the revision
   * @property {Array<string>} [parents] - Array of parent revision IDs
   * @property {string} [branch] - Branch name (usually language code)
   * @property {Array<RevisionTag>} [tags] - Array of revision tags
   * @property {Array<RevisionUrl>} [urls] - Array of action URLs for this revision
   */

  /**
   * @typedef {Object} FormattedRevisionData
   * @property {string} id - Unique identifier for the revision
   * @property {string} message - Revision message or description
   * @property {string} author - Name of the revision author
   * @property {number} timestamp - UNIX timestamp of the revision
   * @property {Array<string>} parents - Array of parent revision IDs
   * @property {Array<string>} refs - Array of branch names (refs)
   * @property {Array<RevisionTag>} tags - Array of revision tags
   * @property {Array<RevisionUrl>} urls - Array of action URLs for this revision
   */

  /**
   * Converts raw commits from the server to the format expected by the RevisionGraph library.
   * 
   * @param {Array<RawRevisionData>} commits - Raw commit data from the server
   * @returns {Array<FormattedRevisionData>} - Formatted commits for the graph library
   * 
   * @example
   * // Convert raw data from API:
   * const rawCommits = await fetchCommits();
   * const formattedCommits = Drupal.revisionGraph.formatCommits(rawCommits);
   */
  Drupal.revisionGraph.formatCommits = function(commits) {
    if (!commits || !commits.length) {
      return [];
    }

    return commits.map(commit => ({
      id: commit.id,
      message: commit.message,
      author: commit.author,
      timestamp: commit.timestamp,
      parents: commit.parents || [],
      refs: commit.branch ? [commit.branch] : [],
      tags: commit.tags || [],
      urls: commit.urls || []
    }));
  };

  /**
   * @typedef {Object} GraphConfig
   * @property {number} width - Width of the graph in pixels
   * @property {number} nodeHeight - Height of each node in pixels
   * @property {number} nodeWidth - Width of each node in pixels
   * @property {number} columnWidth - Width of each column in pixels
   * @property {Array<string>} colors - Array of hex color codes for branches
   * @property {boolean} showLabels - Whether to show branch labels
   * @property {string} theme - Theme name ('light' or 'dark')
   * @property {Map<string, string>} branchColors - Map of branch names to colors
   * @property {Object} uiText - Text strings for UI elements
   */

  /**
   * Creates the revision graph library configuration with all display settings.
   * 
   * @returns {GraphConfig} - Configuration for the RevisionGraph library
   * 
   * @example
   * // Initialize graph with custom configuration
   * const config = Drupal.revisionGraph.createGraphConfig();
   * config.theme = 'dark';
   * config.nodeHeight = 30;
   * const graph = new RevisionGraph(config);
   */
  Drupal.revisionGraph.createGraphConfig = function() {
    return {
      width: 720,
      nodeHeight: 40,
      nodeWidth: 20,
      columnWidth: 50,
      colors: [
        "#A43631", // Red
        "#D07E3F", // Orange
        "#6209D4", // Purple
        "#08C6CD", // Teal
        "#E136A7", // Pink
        "#076CFD", // Blue
        "#8E4299"  // Violet
      ],
      showLabels: true,
      theme: "light",
      branchColors: new Map(),
      // UI text customization with translations
      uiText: {
        revisionTerm: Drupal.t("Revision"),
        historyTerm: Drupal.t("Revision history"),
        graphVisualizationLabel: Drupal.t("Revision graph visualization"),
        interactiveGraphLabel: Drupal.t("Interactive revision graph"),
        graphDescription: Drupal.t("Visual representation of content revision history"),
        detailsLabel: Drupal.t("Revision details"),
        closeButtonLabel: Drupal.t("Close revision details"),
        actionsLabel: Drupal.t("Revision actions"),
        historyListLabel: Drupal.t("Revision history")
      }
    };
  };

  /**
   * Processes commit data to establish parent-child relationships for graph visualization.
   * Establishes the relationships necessary for drawing a connected graph.
   * 
   * @param {Array<RawRevisionData>} commits - Array of commit objects
   * @returns {Array<RawRevisionData>} - Processed commits in chronological order (newest first)
   * 
   * @example
   * // Process commits to establish relationships:
   * const sortedCommits = commits.sort((a, b) => parseInt(a.id, 10) - parseInt(b.id, 10));
   * const processedCommits = Drupal.revisionGraph.processCommits(sortedCommits);
   */
  Drupal.revisionGraph.processCommits = function(commits) {
    if (!commits || !commits.length) {
      return [];
    }

    // Track head commits for each branch
    const branchSpecificHeadCommit = {};
    let previousCommit = null;

    // Process each commit to establish parent relationships
    for (const commit of commits) {
      if (previousCommit) {
        if (previousCommit.branch !== commit.branch && branchSpecificHeadCommit[commit.branch]) {
          // If branch changed and we have a head commit for this branch, link to it
          if (!commit.parents) {
            commit.parents = [];
          }
          commit.parents.push(branchSpecificHeadCommit[commit.branch].id);
        } else {
          // Otherwise link to previous commit in sequence
          if (!commit.parents) {
            commit.parents = [];
          }
          commit.parents.push(previousCommit.id);
        }
      }

      // Update head commit for this branch
      branchSpecificHeadCommit[commit.branch] = commit;
      previousCommit = commit;
    }

    // Reverse the array to get chronological order (newest first)
    return [...commits].reverse();
  };

  /**
   * @typedef {Object} FetcherOptions
   * @property {number} total - Total number of items to fetch
   * @property {number} [offset=0] - Starting offset for fetching
   * @property {number} [limit=50] - Batch size for each fetch
   * @property {Function} [onComplete] - Optional callback when fetching is complete
   */

  /**
   * Class representing a data fetcher with progress indicator.
   * Handles batch loading of revisions with visual feedback on progress.
   */
  class RevisionFetcher {
    /**
     * Creates a new RevisionFetcher instance.
     * 
     * @param {HTMLElement} container - DOM element to attach progress bar to
     * @param {FetcherOptions} options - Configuration options
     * 
     * @example
     * // Create a new fetcher and start loading data:
     * const container = document.getElementById('js-revision-graph');
     * const options = {
     *   total: 100,
     *   limit: 25,
     *   offset: 0
     * };
     * const fetcher = new RevisionFetcher(container, options);
     * fetcher.fetch()
     *   .then(commits => {
     *     console.log(`Loaded ${commits.length} revisions`);
     *   });
     */
    constructor(container, options) {
      this.container = container;
      this.options = options;
      this.data = [];
      this.currentOffset = options.offset || 0;
      this.setupProgressBar();
    }

    /**
     * Sets up progress bar UI elements.
     * Creates and initializes the progress container, bar and text elements.
     * 
     * @private
     */
    setupProgressBar() {
      // Create progress container
      this.progressContainer = document.createElement("div");
      this.progressContainer.className = "revision-graph__progress";

      // Create progress bar
      this.progressBar = document.createElement("div");
      this.progressBar.className = "revision-graph__progress-bar";

      // Create progress text
      this.progressText = document.createElement("div");
      this.progressText.className = "revision-graph__progress-text";
      this.progressText.textContent = Drupal.t("Loading revisions: 0/@total (0%)", {"@total": this.options.total});

      // Assemble progress elements
      this.progressContainer.appendChild(this.progressBar);
      this.progressContainer.appendChild(this.progressText);
    }

    /**
     * Updates the progress bar display based on current progress.
     * Changes colors based on progress thresholds and updates text.
     * 
     * @param {number} current - Current number of items loaded
     * @param {number} total - Total number of items to load
     * @private
     */
    updateProgress(current, total) {
      const percentage = Math.min(Math.round((current / total) * 100), 100);
      this.progressBar.style.width = `${percentage}%`;

      // Show both percentage and item counts
      this.progressText.textContent = Drupal.t("Loading revisions: @current/@total (@percent%)", {
        "@current": current,
        "@total": total,
        "@percent": percentage
      });

      // Add visual styles based on progress
      const thresholds = Drupal.revisionGraph.config.colorThresholds;
      this.progressBar.className = "revision-graph__progress-bar";

      if (percentage < thresholds.low) {
        this.progressBar.classList.add("revision-graph__progress-bar--low");
      } else if (percentage < thresholds.medium) {
        this.progressBar.classList.add("revision-graph__progress-bar--medium");
      } else {
        this.progressBar.classList.add("revision-graph__progress-bar--high");
      }
    }

    /**
     * Shows an error message when fetching fails.
     * 
     * @param {string} message - Error message to display
     * @private
     */
    showError(message) {
      const errorMsg = document.createElement("div");
      errorMsg.className = "revision-graph__error";
      errorMsg.textContent = message;
      this.container.appendChild(errorMsg);
    }

    /**
     * Initiates the data fetching process.
     * Handles pagination, progress updates, and error handling.
     * 
     * @returns {Promise<Array<RawRevisionData>>} Promise resolving to fetched data when complete
     */
    fetch() {
      return new Promise((resolve, reject) => {
        // Show progress bar at start
        this.container.appendChild(this.progressContainer);

        /**
         * Recursive function to fetch data in batches
         * @private
         */
        const fetchBatch = () => {
          this.updateProgress(this.data.length, this.options.total);

          window.fetch(`revision_graph_items?offset=${this.currentOffset}&limit=${this.options.limit}`)
            .then(response => {
              if (!response.ok) {
                throw new Error(`HTTP error ${response.status}`);
              }
              return response.json();
            })
            .then(data => {
              // Store the fetched data
              this.data = [...this.data, ...data];

              // Update progress bar
              this.updateProgress(this.data.length, this.options.total);

              // If we need more items and received data, fetch the next batch
              if (this.data.length < this.options.total && data.length > 0) {
                this.currentOffset += data.length;
                // Use setTimeout to prevent UI blocking on rapid fetches
                setTimeout(fetchBatch, Drupal.revisionGraph.config.fetchDelay);
              } else {
                // All data fetched, remove progress bar
                if (this.container.contains(this.progressContainer)) {
                  this.container.removeChild(this.progressContainer);
                }
                resolve(this.data);
              }
            })
            .catch(error => {
              console.error("Error fetching revisions:", error);

              // Remove progress bar on error
              if (this.container.contains(this.progressContainer)) {
                this.container.removeChild(this.progressContainer);
              }

              // Show error message
              this.showError(Drupal.t("Error loading revisions. Please try again later."));
              reject(error);
            });
        };

        // Start fetching
        fetchBatch();
      });
    }
  }

  /**
   * Drupal behavior to initialize the revision graph.
   * Sets up the data fetching, processing and rendering pipeline.
   */
  Drupal.behaviors.revisionGraph = {
    /**
     * Attaches the behavior to create and initialize the revision graph.
     * 
     * @param {Object} context - The context for Drupal behaviors
     */
    attach: function (context) {
      // Initialize graphs on all matching elements
      once('revision-graph', '#js-revision-graph', context).forEach(function (element) {
        // Extract configuration from data attributes
        const config = {
          total: parseInt(element.dataset.total, 10) || 0,
          offset: parseInt(element.dataset.page, 10) * parseInt(element.dataset.limit, 10) || 0,
          limit: parseInt(element.dataset.limit, 10) || 50
        };

        // Create fetcher and get data
        const fetcher = new RevisionFetcher(element, config);
        fetcher.fetch()
          .then(commits => {
            // Sort commits by ID
            const sortedCommits = commits.sort((a, b) => parseInt(a.id, 10) - parseInt(b.id, 10));

            // Process parent-child relationships
            const processedCommits = Drupal.revisionGraph.processCommits(sortedCommits);

            // Format for graph library
            const graphCommits = Drupal.revisionGraph.formatCommits(processedCommits);

            // Get graph containers
            const graphNode = document.getElementById("js-revision-graph--graph");
            const listNode = document.getElementById("js-revision-graph--list");

            if (!graphNode || !listNode) {
              console.error("Missing graph containers");
              return;
            }

            // Initialize the graph with configuration
            const graph = new RevisionGraph(Drupal.revisionGraph.createGraphConfig());

            // Render graph and list views
            const graphGenerator = graph.renderGraph(graphNode, graphCommits);
            const listGenerator = graph.renderList(listNode, graphCommits);
          })
          .catch(error => {
            console.error("Failed to initialize revision graph:", error);
          });
      });
    }
  };

})(Drupal, once);

