# Revision Graph

A lightweight, customizable Git commit graph visualization library for Drupal. This library provides an interactive visualization of Git commit history with branch and merge visualization.

## Features

- Interactive Git commit graph visualization
- Branch and merge visualization
- Customizable styling and theming
- Configurable UI terminology
- TypeScript support
- Lightweight and performant
- No external dependencies

## Installation

TODO: The code is yet to be distributed via npm. Hence following command doesn't
work as of yet.

```bash
npm install revision-graph
```

## Usage

### Basic Setup

```typescript
import { RevisionGraph } from "revision-graph";

// Create a container element
const container = document.getElementById("revision-graph");

// Initialize the graph with options
const graph = new RevisionGraph({
  width: 800,
  nodeHeight: 40,
  nodeWidth: 20,
  columnWidth: 50,
  colors: ["#e11d21", "#fbca04", "#009800", "#006b75", "#207de5", "#0052cc", "#5319e7"],
  showLabels: true,
  theme: "light"
});

// Render the graph
const graphGenerator = graph.renderGraph(container, commits);

// Render the commit list
const listGenerator = graph.renderList(container, commits);
```

### Customizing UI Terminology

The library supports customization of UI text to match your preferred terminology. This is especially useful when adapting the interface to specific content management systems like Drupal, where "revision" might be preferred over "commit".

```typescript
const graph = new RevisionGraph({
  // Standard options
  width: 800,
  theme: "light",
  
  // UI text customization
  uiText: {
    revisionTerm: "Revision",                 // Term for a single content state (default: "Commit")
    historyTerm: "Revision history",          // Term for content history (default: "Git commit history")
    graphVisualizationLabel: "Revision graph visualization",
    interactiveGraphLabel: "Interactive revision graph",
    graphDescription: "Visual representation of content revision history",
    detailsLabel: "Revision details",         // Label for popup (default: "Commit details")
    closeButtonLabel: "Close revision details",
    actionsLabel: "Revision actions",         // Label for actions dropdown (default: "Commit actions")
    historyListLabel: "Revision history"      // Label for list view (default: "Revision history")
  }
});
```

The default terminology in the library now uses "Revision" instead of "Commit" to better align with Drupal's content management concepts.

### Type Definitions

```typescript
interface Commit {
  id: string;
  message: string;
  author: string;
  timestamp: number;
  parents: string[];
  refs?: string[];
  tags?: Tag[];
  urls?: CommitUrl[];
}

interface Tag {
  name: string;
  color: string;
  label: string;
}

interface CommitUrl {
  label: string;
  url: string;
}

interface GraphOptions {
  width: number;
  nodeHeight: number;
  nodeWidth: number;
  columnWidth: number;
  colors: string[];
  showLabels: boolean;
  theme: "light" | "dark";
  uiText?: UITextOptions;
}

interface UITextOptions {
  revisionTerm?: string;
  historyTerm?: string;
  graphVisualizationLabel?: string;
  interactiveGraphLabel?: string;
  graphDescription?: string;
  detailsLabel?: string;
  closeButtonLabel?: string;
  actionsLabel?: string;
  historyListLabel?: string;
}
```

## Development

### Prerequisites

- Node.js (v14 or higher)
- npm (v6 or higher)

### Setup

This JavaScript project is part of revision_graph module on Drupal.org.
Hence all develpoment happens and is managed via Drupal.org.

1. Clone the main repository:
```bash
git clone https://git.drupalcode.org/project/revision_graph.git
cd revision-graph/revision-graph-js
```

2. Install dependencies:
```bash
npm install
```

3. Build the library:
```bash
npm run build
```

### Available Scripts

- `npm run build` - Build the library
- `npm run lint` - Run ESLint
- `npm run type-check` - Run TypeScript type checking
- `npm test` - Run tests

