import type { Commit, GraphOptions, GraphNode, Connection, UITextOptions } from "./types";
import "./styles.css";

interface ScrollContainer extends HTMLDivElement {
    ignoreScrollEvent?: boolean;
}

interface GraphPoint {
    x: number;
    y: number;
    node: string;
}

// Define the ActionLink type
interface ActionLink {
    url: string;
    label?: string;
}

/**
 * Default options for the graph generator
 */
const DEFAULT_OPTIONS: GraphOptions = {
    width: 800,
    nodeHeight: 40,
    nodeWidth: 20,
    columnWidth: 50,
    colors: ["#e11d21", "#fbca04", "#009800", "#006b75", "#207de5", "#0052cc", "#5319e7"],
    showLabels: true,
    theme: "light",
    branchColors: new Map(),
};

/**
 * Default UI text options
 */
const DEFAULT_UI_TEXT: UITextOptions = {
    revisionTerm: "Commit",
    historyTerm: "Git commit history",
    graphVisualizationLabel: "Git commit graph visualization",
    interactiveGraphLabel: "Interactive commit graph",
    graphDescription: "Visual representation of Git commit history showing branches and connections between commits",
    detailsLabel: "Commit details",
    closeButtonLabel: "Close commit details",
    actionsLabel: "Commit actions",
    historyListLabel: "Revision history"
};

/**
 * RevisionGraphGenerator class for creating visual representations of Git commit history
 * @example
 * ```typescript
 * const generator = new RevisionGraphGenerator(options);
 * const nodes = generator.processCommits(commits);
 * ```
 */
export class RevisionGraphGenerator {
    private readonly svg: SVGSVGElement;
    private readonly svgns: string = "http://www.w3.org/2000/svg";
    private readonly options: GraphOptions;
    private readonly uiText: UITextOptions;
    private readonly container: HTMLElement;
    private readonly graphContainer: ScrollContainer;
    private readonly branchColors: Map<string, string> = new Map();
    private readonly positionCache: Map<string, GraphPoint> = new Map();
    private readonly nodeMap: Map<string, GraphNode> = new Map();
    private readonly eventListeners: Map<string, (e: Event) => void> = new Map();
    private popup: HTMLDivElement | null = null;
    private connectorLine: SVGPathElement | null = null;
    private selectedNodeIndex: number = -1;
    private readonly POPUP_OFFSET = 40;
    private readonly POPUP_WIDTH = 300;
    private clickOutsideHandler: ((e: MouseEvent) => void) | null = null;
    private readonly BRANCH_LABEL_HEIGHT = 50; // Height reserved for branch labels
    private activeNodeId: string | null = null; // Track active node
    private previouslyFocusedElement: HTMLElement | null = null;
    private activeNodePosition: GraphPoint | null = null; // Store active node position
    private scrollHandler: ((e: Event) => void) | null = null; // Scroll event handler

    private readonly styles = {
        container: "revision-graph",
        graph: "revision-graph__container"
    } as const;

    /**
     * Creates a new instance of RevisionGraphGenerator
     * @param container - The container element to render the graph in
     * @param options - Configuration options for the graph
     */
    constructor(container: HTMLElement, options: GraphOptions) {
        this.options = { ...DEFAULT_OPTIONS, ...options };
        this.uiText = { ...DEFAULT_UI_TEXT, ...options.uiText };
        this.container = container;

        // Apply container styles and ARIA attributes
        this.container.className = this.styles.container;
        this.container.setAttribute("role", "region");
        this.container.setAttribute("aria-label", `${this.uiText.graphVisualizationLabel || DEFAULT_UI_TEXT.graphVisualizationLabel}`);

        // Create graph container with accessibility attributes
        this.graphContainer = document.createElement("div");
        this.graphContainer.className = this.styles.graph;
        this.graphContainer.setAttribute("role", "application");
        this.graphContainer.setAttribute("aria-label", `${this.uiText.interactiveGraphLabel || DEFAULT_UI_TEXT.interactiveGraphLabel}`);
        this.graphContainer.style.width = `${this.options.width}px`;
        this.container.appendChild(this.graphContainer);

        // Create SVG element with accessibility support
        this.svg = document.createElementNS(this.svgns, "svg") as SVGSVGElement;
        this.svg.setAttribute("width", "100%");
        this.svg.setAttribute("height", "100%");
        this.svg.style.overflow = "visible";
        this.svg.setAttribute("role", "img");
        this.svg.setAttribute("aria-label", `${this.uiText.graphVisualizationLabel || DEFAULT_UI_TEXT.graphVisualizationLabel}`);
        this.graphContainer.appendChild(this.svg);

        // Add description for screen readers
        const desc = document.createElementNS(this.svgns, "desc");
        desc.textContent = `${this.uiText.graphDescription || DEFAULT_UI_TEXT.graphDescription}`;
        this.svg.appendChild(desc);

        // Set theme if provided
        if (options.theme === "dark") {
            this.container.setAttribute("data-theme", "dark");
        }

        // Add keyboard event listeners
        this.setupKeyboardNavigation();
    }

    /**
     * Sets up keyboard navigation for the graph
     */
    private setupKeyboardNavigation(): void {
        this.graphContainer.addEventListener("keydown", (e: KeyboardEvent) => {
            if (!this.selectedNodeIndex) return;

            const nodes = Array.from(this.svg.querySelectorAll(".commit-node"));
            let newIndex = this.selectedNodeIndex;

            switch (e.key) {
                case "ArrowUp":
                    newIndex = Math.max(0, this.selectedNodeIndex - 1);
                    break;
                case "ArrowDown":
                    newIndex = Math.min(nodes.length - 1, this.selectedNodeIndex + 1);
                    break;
                case "Home":
                    newIndex = 0;
                    break;
                case "End":
                    newIndex = nodes.length - 1;
                    break;
                case "Escape":
                    if (this.popup) {
                        this.hidePopup();
                    }
                    return;
                default:
                    return;
            }

            if (newIndex !== this.selectedNodeIndex) {
                e.preventDefault();
                const node = nodes[newIndex] as SVGElement;
                const nodeId = node.dataset.nodeId;
                if (nodeId) {
                    const graphNode = this.nodeMap.get(nodeId);
                    if (graphNode) {
                        const pos = this.positionCache.get(nodeId);
                        if (pos) {
                            this.showPopup(pos, graphNode);
                            this.selectedNodeIndex = newIndex;
                            node.focus();
                        }
                    }
                }
            }
        });
    }


    /**
     * Generates and renders the graph with the provided commits
     * @param commits - Array of commits to convert and render
     */
    generate(commits: Commit[]): void {
        const nodes = this.convertCommitsToNodes(commits);
        this.renderNodes(nodes);
    }

    /**
     * Renders the graph nodes
     * @param nodes - Array of graph nodes to render
     */
    private renderNodes(nodes: GraphNode[]): void {
        // Clear existing content
        this.svg.innerHTML = "";
        this.positionCache.clear();

        // Sort nodes by timestamp
        nodes.sort((a, b) => b.timestamp - a.timestamp);

        // Calculate positions with offset for branch labels
        nodes.forEach((node, index) => {
            const x = node.column * this.options.columnWidth + (this.options.columnWidth / 2);
            const y = this.BRANCH_LABEL_HEIGHT + (index * this.options.nodeHeight) + (this.options.nodeHeight / 2);
            this.positionCache.set(node.id, { x, y, node: node.id });
        });

        // Draw column grid lines first (behind everything)
        this.drawColumnGrid(nodes);
        
        // Draw date markers
        this.drawDateMarkers(nodes);

        // Draw branch column indicators
        this.drawBranchColumns(nodes);

        // Draw connections
        nodes.forEach(node => {
            node.connections.forEach(connection => {
                const from = this.positionCache.get(connection.from.id);
                const to = this.positionCache.get(connection.to.id);
                if (from && to) {
                    this.drawConnection(from, to, connection.color, connection.type);
                }
            });
        });

        // Draw nodes on top
        nodes.forEach(node => {
            const pos = this.positionCache.get(node.id);
            if (pos) {
                this.drawNode(pos, node);
            }
        });
    }

    /**
     * Draws vertical grid lines for columns
     */
    private drawColumnGrid(nodes: GraphNode[]): void {
        // Find the maximum column number
        const maxColumn = Math.max(...nodes.map(node => node.column));
        
        // Find the maximum y position
        const maxY = Math.max(...Array.from(this.positionCache.values()).map(pos => pos.y));
        
        // Draw vertical lines for each column
        for (let i = 0; i <= maxColumn; i++) {
            const x = i * this.options.columnWidth + (this.options.columnWidth / 2);
            const line = document.createElementNS(this.svgns, "line");
            
            line.setAttribute("x1", x.toString());
            line.setAttribute("y1", "0");
            line.setAttribute("x2", x.toString());
            line.setAttribute("y2", (maxY + this.options.nodeHeight).toString());
            line.setAttribute("class", "revision-graph__grid-line");
            
            // Insert the grid line as the first child to ensure it's behind everything
            this.svg.insertBefore(line, this.svg.firstChild);
        }
    }

    /**
     * Draws branch column indicators
     */
    private drawBranchColumns(nodes: GraphNode[]): void {
        const branchColumns = new Map<number, string>();
        const columnColors = new Map<number, string>();

        // Find the first node in each column that has a branch ref
        nodes.forEach(node => {
            if (node.refs && node.refs.length > 0 && !branchColumns.has(node.column)) {
                branchColumns.set(node.column, node.refs[0]);
                columnColors.set(node.column, node.color || this.options.colors[0]);
            }
        });

        // Draw branch indicators
        branchColumns.forEach((branchName, column) => {
            const x = column * this.options.columnWidth + (this.options.columnWidth / 2);
            const color = columnColors.get(column) || this.options.colors[0];

            // Draw branch label
            const label = document.createElementNS(this.svgns, "text") as SVGTextElement;
            label.setAttribute("x", x.toString());
            label.setAttribute("y", (this.BRANCH_LABEL_HEIGHT - 15).toString());
            label.setAttribute("text-anchor", "middle");
            label.setAttribute("fill", "#ffffff");
            label.setAttribute("class", "revision-graph__branch-label");
            label.textContent = branchName;
            this.svg.appendChild(label);

            // Draw background pill for better readability
            const textWidth = (label.getComputedTextLength?.() ?? branchName.length * 8);
            const pill = document.createElementNS(this.svgns, "rect");
            pill.setAttribute("x", (x - textWidth/2 - 4).toString());
            pill.setAttribute("y", (this.BRANCH_LABEL_HEIGHT - 30).toString());
            pill.setAttribute("width", (textWidth + 8).toString());
            pill.setAttribute("height", "20");
            pill.setAttribute("rx", "2");
            pill.setAttribute("fill", color);
            pill.setAttribute("class", "revision-graph__branch-background");
            this.svg.insertBefore(pill, label);
        });
    }

    /**
     * Draws a connection line between two points
     */
    private drawConnection(from: GraphPoint, to: GraphPoint, color: string, type?: "direct" | "bridge"): void {
        const path = document.createElementNS(this.svgns, "path");
        const midY = (from.y + to.y) / 2;
        
        // Calculate control points for the curve
        const dy = Math.abs(to.y - from.y);
        const curveStrength = Math.min(dy * 0.5, 50); // Limit the curve strength
        
        let d: string;
        if (type === "bridge") {
            // For branch transitions, use a more pronounced curve
            d = `M ${from.x} ${from.y} 
                 C ${from.x} ${from.y + curveStrength},
                   ${to.x} ${to.y - curveStrength},
                   ${to.x} ${to.y}`;
        } else {
            // For direct connections, use a gentler curve
            const controlX = Math.abs(to.x - from.x) * 0.1; // Small horizontal curve
            d = `M ${from.x} ${from.y} 
                 C ${from.x + controlX} ${from.y + curveStrength},
                   ${to.x - controlX} ${to.y - curveStrength},
                   ${to.x} ${to.y}`;
        }

        path.setAttribute("d", d);
        path.setAttribute("stroke", color);
        path.setAttribute("stroke-width", "2");
        path.setAttribute("fill", "none");
        path.setAttribute("class", `revision-graph__connection ${type || "direct"}`);
        
        this.svg.appendChild(path);
    }

    /**
     * Draws a commit node
     */
    private drawNode(pos: GraphPoint, node: GraphNode): void {
        const group = document.createElementNS(this.svgns, "g") as SVGGElement;
        group.setAttribute("class", "revision-graph__node-group");
        group.setAttribute("role", "button");
        group.setAttribute("tabindex", "0");
        group.setAttribute("aria-label", `${this.uiText.revisionTerm || DEFAULT_UI_TEXT.revisionTerm} ${node.id.substring(0, 7)} by ${node.author}: ${node.message.split("\n")[0]}`);
        group.setAttribute("data-node-id", node.id);
        
        if (node.tags?.length) {
            group.setAttribute("aria-description", `Has tags: ${node.tags.map(t => t.name).join(", ")}`);
        }

        // Draw highlight background for active node
        if (node.id === this.activeNodeId && this.popup) {
            const highlight = document.createElementNS(this.svgns, "rect");
            const padding = 10;
            highlight.setAttribute("x", (pos.x - this.options.columnWidth/2 - padding).toString());
            highlight.setAttribute("y", (pos.y - this.options.nodeHeight/2).toString());
            highlight.setAttribute("width", (this.options.columnWidth + padding * 2).toString());
            highlight.setAttribute("height", this.options.nodeHeight.toString());
            highlight.setAttribute("class", "revision-graph__highlight");
            highlight.setAttribute("rx", "4");
            group.appendChild(highlight);
        }

        // Draw outer circle for tags
        if (node.tags && node.tags.length > 0) {
            node.tags.forEach(tag => {
                const outerCircle = document.createElementNS(this.svgns, "circle");
                outerCircle.setAttribute("cx", pos.x.toString());
                outerCircle.setAttribute("cy", pos.y.toString());
                outerCircle.setAttribute("r", (this.options.nodeWidth / 2 + 4).toString());
                outerCircle.setAttribute("fill", "none");
                outerCircle.setAttribute("class", `revision-graph__tag-indicator revision-graph__tag-indicator--${tag.name}`);
                group.appendChild(outerCircle);
            });
        }

        // Draw main commit circle
        const circle = document.createElementNS(this.svgns, "circle");
        circle.setAttribute("cx", pos.x.toString());
        circle.setAttribute("cy", pos.y.toString());
        circle.setAttribute("r", (this.options.nodeWidth / 2).toString());
        circle.setAttribute("fill", node.color || this.options.colors[0]);
        circle.setAttribute("stroke", "#ffffff");
        circle.setAttribute("stroke-width", "2");
        
        const classList = ["revision-graph__node"];
        if (node.id === this.activeNodeId) {
            classList.push("revision-graph__node--active");
            group.setAttribute("aria-expanded", "true");
        }
        
        circle.setAttribute("class", classList.join(" "));

        // Add event listeners with proper type casting
        group.addEventListener("click", ((e: Event) => {
            e.stopPropagation();
            this.showPopup(pos, node);
        }) as EventListener);

        group.addEventListener("keydown", ((e: Event) => {
            const keyEvent = e as KeyboardEvent;
            if (keyEvent.key === "Enter" || keyEvent.key === " ") {
                e.preventDefault();
                this.showPopup(pos, node);
            }
        }) as EventListener);

        group.appendChild(circle);
        this.svg.appendChild(group);
    }

    /**
     * Shows a popup with commit details
     */
    private showPopup(pos: GraphPoint, node: GraphNode): void {
        // Check if we're switching to a different node
        const isSwitchingNodes = this.activeNodeId !== null && this.activeNodeId !== node.id;
        
        // Remove existing popup and connector
        this.hidePopup();

        // Store active node position for scroll updates
        this.activeNodePosition = { ...pos };
        this.activeNodeId = node.id;

        // Create new popup using CSS classes instead of inline styles
        this.popup = document.createElement("div");
        this.popup.className = "revision-graph__popup";
        this.popup.setAttribute("role", "dialog");
        this.popup.setAttribute("aria-label", `${this.uiText.detailsLabel || DEFAULT_UI_TEXT.detailsLabel}`);
        this.popup.setAttribute("aria-modal", "true");
        this.popup.setAttribute("tabindex", "-1"); // Make focusable but not in tab order

        // Create close button
        const closeButton = document.createElement("button");
        closeButton.className = "revision-graph__popup-close";
        closeButton.setAttribute("aria-label", `${this.uiText.closeButtonLabel || DEFAULT_UI_TEXT.closeButtonLabel}`);
        closeButton.innerHTML = "×";
        closeButton.addEventListener("click", () => this.hidePopup());
        this.popup.appendChild(closeButton);

        // Create header
        const header = document.createElement("div");
        header.className = "revision-graph__popup-header";
        header.setAttribute("role", "heading");
        header.setAttribute("aria-level", "2");

        // Add branch indicator if present
        if (node.refs && node.refs.length > 0) {
          const branchLabel = document.createElement("div");
          branchLabel.className = "revision-graph__branch-label";
          branchLabel.style.backgroundColor = this.getBranchColor(node.refs[0]);
          branchLabel.textContent = node.refs[0];
          header.appendChild(branchLabel);
        }

        const headerContent = document.createElement("div");
        headerContent.className = "revision-graph__popup-content";

        const title = document.createElement("div");
        title.className = "revision-graph__popup-title";
        title.setAttribute("id", "commit-popup-title");
        title.innerHTML = node.message;
        headerContent.appendChild(title);

        const hash = document.createElement("div");
        hash.className = "revision-graph__popup-hash";
        hash.setAttribute("role", "text");
        hash.setAttribute("aria-label", `${this.uiText.revisionTerm || DEFAULT_UI_TEXT.revisionTerm} hash`);
        hash.textContent = node.id.substring(0, 8); // Show 8 characters of hash
        headerContent.appendChild(hash);

        header.appendChild(headerContent);
        this.popup.appendChild(header);

        // Add metadata
        const meta = document.createElement("div");
        meta.className = "revision-graph__popup-meta";
        meta.setAttribute("role", "group");
        meta.setAttribute("aria-label", `${this.uiText.revisionTerm || DEFAULT_UI_TEXT.revisionTerm} metadata`);
        

        // Author
        const authorContainer = document.createElement("div");
        authorContainer.style.display = "flex";
        authorContainer.style.alignItems = "center";
        authorContainer.style.marginRight = "12px";
        
        const author = document.createElement("span");
        author.className = "revision-graph__popup-author";
        author.setAttribute("role", "text");
        author.setAttribute("aria-label", `Author: ${node.author}`);
        author.textContent = node.author;
        authorContainer.appendChild(author);
        
        meta.appendChild(authorContainer);

        // Date
        const dateContainer = document.createElement("div");
        dateContainer.style.display = "flex";
        dateContainer.style.alignItems = "center";
        
        const date = document.createElement("span");
        date.className = "revision-graph__popup-date";
        date.setAttribute("role", "text");
        const commitDate = new Date(node.timestamp * 1000);
        date.setAttribute("aria-label", `${this.uiText.revisionTerm || DEFAULT_UI_TEXT.revisionTerm} on ${commitDate.toLocaleString()}`);
        date.textContent = commitDate.toLocaleString();
        dateContainer.appendChild(date);
        
        meta.appendChild(dateContainer);

        this.popup.appendChild(meta);

        // Add tags if present
        if (node.tags && node.tags.length > 0) {
            const labels = document.createElement("div");
            labels.className = "revision-graph__popup-labels";
            labels.setAttribute("role", "group");
            labels.setAttribute("aria-label", `${this.uiText.revisionTerm || DEFAULT_UI_TEXT.revisionTerm} tags`);

            // Only add tags, not branch refs
            node.tags.forEach(tag => {
                const tagElement = document.createElement("span");
                tagElement.className = "revision-graph__tag";
                tagElement.setAttribute("role", "text");
                tagElement.setAttribute("aria-label", `Tag: ${tag.name}`);
                tagElement.textContent = tag.label;
                
                // Set tag colors
                if (tag.color) {
                    tagElement.style.backgroundColor = `${tag.color}15`;
                    tagElement.style.color = tag.color;
                    tagElement.style.borderColor = tag.color;
                }
                
                labels.appendChild(tagElement);
            });

            this.popup.appendChild(labels);
        }

        // Add URL menu
        const urlMenu = document.createElement("ul");
        urlMenu.className = "revision-graph__popup-menu";

        node.urls.forEach((actionLink: ActionLink) => {
            const menuItem = document.createElement("li");
            menuItem.className = "revision-graph__popup-menu-item";

            const link = document.createElement("a");
            link.href = actionLink.url;
            link.className = "revision-graph__popup-menu-link";
            link.textContent = actionLink.label || actionLink.url;
            link.target = "_blank";
            link.rel = "noopener noreferrer";
            
            menuItem.appendChild(link);
            urlMenu.appendChild(menuItem);
        });

        this.popup.appendChild(urlMenu);

        // Position popup
        const rect = this.svg.getBoundingClientRect();
        const scrollTop = window.pageYOffset || document.documentElement.scrollTop;
        const scrollLeft = window.pageXOffset || document.documentElement.scrollLeft;

        let x = rect.left + pos.x + this.POPUP_OFFSET + scrollLeft;
        let y = rect.top + pos.y - this.POPUP_OFFSET + scrollTop;

        // Adjust position if popup would go off screen
        const viewportWidth = window.innerWidth;
        const viewportHeight = window.innerHeight;
        const popupRect = { width: 400, height: 300 }; // Updated dimensions

        if (x + popupRect.width > viewportWidth) {
            x = x - popupRect.width - (this.POPUP_OFFSET * 2);
        }

        if (y + popupRect.height > viewportHeight) {
            y = y - popupRect.height;
        }

        this.popup.style.left = `${x}px`;
        this.popup.style.top = `${y}px`;

        // Add to DOM and set focus
        document.body.appendChild(this.popup);
        
        // Store previously focused element
        this.previouslyFocusedElement = document.activeElement as HTMLElement;
        
        // Focus the popup
        this.popup.focus();

        // Calculate popup center point for connector
        const popupCenterPoint = {
            x: x - rect.left + (x > pos.x ? 0 : popupRect.width),
            y: y - rect.top + popupRect.height / 2
        };

        // Draw connector line
        this.drawConnectorLine(pos, popupCenterPoint, node.color || this.options.colors[0]);

        // Add scroll event listener to update connector line
        this.scrollHandler = (() => {
            if (this.activeNodePosition && this.popup) {
                const updatedRect = this.svg.getBoundingClientRect();
                
                // Get current popup position
                const popupRect = this.popup.getBoundingClientRect();
                const popupCenter = {
                    x: popupRect.left + (popupRect.width / 2) - updatedRect.left,
                    y: popupRect.top + (popupRect.height / 2) - updatedRect.top
                };
                
                // Redraw connector line with updated positions
                this.drawConnectorLine(
                    this.activeNodePosition,
                    popupCenter,
                    node.color || this.options.colors[0]
                );
            }
        }) as EventListener;
        
        window.addEventListener("scroll", this.scrollHandler);
        
        // Also listen for resize events to update the connector
        const resizeHandler = this.scrollHandler; // Reuse the same handler
        window.addEventListener("resize", resizeHandler);
        
        // Store the resize handler to remove it later
        this.eventListeners.set("resize", resizeHandler);

        // Add click outside handler and keyboard navigation
        this.setupPopupInteractions();
    }

    /**
     * Sets up popup interactions including keyboard navigation and click outside
     */
    private setupPopupInteractions(): void {
        if (!this.popup) return;

        // Add keyboard navigation
        this.popup.addEventListener("keydown", (e: KeyboardEvent) => {
            switch (e.key) {
                case "Escape":
                    e.preventDefault();
                    this.hidePopup();
                    break;
                case "Tab":
                    this.handlePopupTabKey(e);
                    break;
            }
        });

        // Set up click outside handler
        setTimeout(() => {
            this.clickOutsideHandler = ((e: Event) => {
                const mouseEvent = e as MouseEvent;
                if (this.popup && !this.popup.contains(mouseEvent.target as Node)) {
                    this.hidePopup();
                }
            }) as EventListener;

            document.addEventListener("click", this.clickOutsideHandler);
        }, 0);
    }

    /**
     * Handles tab key navigation within the popup
     */
    private handlePopupTabKey(e: KeyboardEvent): void {
        if (!this.popup) return;

        const focusableElements = this.popup.querySelectorAll(
            'button, [href], input, select, textarea, [tabindex]:not([tabindex="-1"])'
        );
        const firstElement = focusableElements[0] as HTMLElement;
        const lastElement = focusableElements[focusableElements.length - 1] as HTMLElement;

        if (e.shiftKey) {
            if (document.activeElement === firstElement) {
                e.preventDefault();
                lastElement.focus();
            }
        } else {
            if (document.activeElement === lastElement) {
                e.preventDefault();
                firstElement.focus();
            }
        }
    }

    /**
     * Draws a dashed connector line between commit node and popup
     */
    private drawConnectorLine(from: GraphPoint, to: { x: number; y: number }, color: string): void {
        // Remove existing connector line if present
        if (this.connectorLine) {
            this.connectorLine.remove();
            this.connectorLine = null;
        }
        
        const path = document.createElementNS(this.svgns, "path") as SVGPathElement;
        
        // Use straight lines for a more concise, angular look
        const midX = (from.x + to.x) / 2;
        
        // Create angular path with straight lines
        const d = `M ${from.x} ${from.y} 
                   L ${midX} ${from.y}
                   L ${midX} ${to.y}
                   L ${to.x} ${to.y}`;
        
        path.setAttribute("d", d);
        path.setAttribute("stroke", color);
        path.setAttribute("class", "revision-graph__connector");
        
        this.connectorLine = path;
        this.svg.appendChild(path);
    }

    /**
     * Updates the dimensions of the graph
     */
    updateDimensions(width: number, height: number): void {
        this.graphContainer.style.width = `${width}px`;
        this.graphContainer.style.height = `${height}px`;
    }

    /**
     * Gets the color for a branch
     * @param branch - Branch name
     * @returns Color string
     */
    private getBranchColor(branch: string): string {
        const colors = this.options.colors || ["#222222"];
        const index = Math.abs(this.hashString(branch)) % colors.length;
        return colors[index];
    }
    /**
     * Creates a hash code from a string
     * @param str - String to hash
     * @returns Hash code
     */
    private hashString(str: string): number {
        let hash = 0;
        for (let i = 0; i < str.length; i++) {
            const char = str.charCodeAt(i);
            hash = ((hash << 5) - hash) + char;
            hash = hash & hash;
        }
        return hash;
    }

    /**
     * Cleans up resources and event listeners
     */
    cleanup(): void {
        // Hide popup and remove related event listeners
        this.hidePopup();

        // Remove all registered event listeners
        this.eventListeners.forEach((handler, event) => {
            this.container.removeEventListener(event, handler);
        });
        this.eventListeners.clear();

        // Remove scroll handler if it exists
        if (this.scrollHandler) {
            window.removeEventListener("scroll", this.scrollHandler);
            this.scrollHandler = null;
        }

        // Remove click outside handler if it exists
        if (this.clickOutsideHandler) {
            document.removeEventListener("click", this.clickOutsideHandler);
            this.clickOutsideHandler = null;
        }

        // Clear references
        this.activeNodeId = null;
        this.activeNodePosition = null;
        this.previouslyFocusedElement = null;
    }

    /**
     * Converts commits to graph nodes
     */
    private convertCommitsToNodes(commits: Commit[]): GraphNode[] {
        const nodes: GraphNode[] = [];
        const commitMap = new Map<string, Commit>();
        this.nodeMap.clear(); // Clear existing node map
        const branchColumns = new Map<string, number>();
        let nextColumn = 0;

        // First pass: create nodes and maps
        commits.forEach((commit) => {
            commitMap.set(commit.id, commit);
            const node: GraphNode = {
                id: commit.id,
                message: commit.message,
                author: commit.author,
                timestamp: commit.timestamp,
                parents: commit.parents,
                refs: commit.refs || [],
                tags: commit.tags || [],
                urls: commit.urls || [],
                column: 0,
                connections: [],
                color: "",
            };
            nodes.push(node);
            this.nodeMap.set(commit.id, node);

            // Assign colors to branches
            commit.refs?.forEach((ref) => {
                if (!this.branchColors.has(ref)) {
                    const color = this.getBranchColor(ref);
                    this.branchColors.set(ref, color);
                }
                if (!branchColumns.has(ref)) {
                    branchColumns.set(ref, nextColumn++);
                }
            });
        });

        // Second pass: calculate columns and create connections
        nodes.forEach((node) => {
            const commit = commitMap.get(node.id);
            if (!commit) return;

            // Set column based on branch
            if (commit.refs && commit.refs.length > 0) {
                node.column = branchColumns.get(commit.refs[0]) || 0;
                node.color = this.branchColors.get(commit.refs[0]) || this.options.colors[0];
            } else if (commit.parents.length > 0) {
                // If no branch but has parent, use parent's column
                const parentNode = this.nodeMap.get(commit.parents[0]);
                if (parentNode) {
                    node.column = parentNode.column;
                    node.color = parentNode.color;
                }
            }

            // Create connections to parents
            commit.parents.forEach((parentId) => {
                const parentNode = this.nodeMap.get(parentId);
                if (parentNode) {
                    const connection: Connection = {
                        from: node,
                        to: parentNode,
                        color: node.color || this.options.colors[0],
                        type: "direct"
                    };
                    node.connections.push(connection);

                    // If parent is in a different column, add a bridge connection
                    if (parentNode.column !== node.column) {
                        const bridge: Connection = {
                            from: node,
                            to: parentNode,
                            color: node.color || this.options.colors[0],
                            type: "bridge"
                        };
                        node.connections.push(bridge);
                    }
                }
            });
        });

        return nodes;
    }

    /**
     * Hides the commit detail popup and cleans up related elements
     */
    private hidePopup(): void {
        if (this.popup) {
            document.body.removeChild(this.popup);
            this.popup = null;
        }

        if (this.connectorLine) {
            this.connectorLine.remove();
            this.connectorLine = null;
        }

        if (this.clickOutsideHandler) {
            document.removeEventListener("click", this.clickOutsideHandler);
            this.clickOutsideHandler = null;
        }

        // Remove scroll event listener
        if (this.scrollHandler) {
            window.removeEventListener("scroll", this.scrollHandler);
            this.scrollHandler = null;
        }

        // Remove resize event listener if it exists
        const resizeHandler = this.eventListeners.get("resize");
        if (resizeHandler) {
            window.removeEventListener("resize", resizeHandler);
            this.eventListeners.delete("resize");
        }

        // Reset active node state and position
        if (this.activeNodeId) {
            const activeNode = this.svg.querySelector(`[data-node-id="${this.activeNodeId}"]`);
            if (activeNode) {
                activeNode.classList.remove("revision-graph__node--active");
                activeNode.setAttribute("aria-expanded", "false");
            }
            this.activeNodeId = null;
            this.activeNodePosition = null;
        }

        // Restore focus to previously focused element
        if (this.previouslyFocusedElement) {
            this.previouslyFocusedElement.focus();
            this.previouslyFocusedElement = null;
        }
    }

    /**
     * Draws horizontal date markers to show when commits occurred
     * @param nodes - Array of graph nodes
     */
    private drawDateMarkers(nodes: GraphNode[]): void {
        if (nodes.length === 0) return;
        
        // Find the maximum column number to determine width
        const maxColumn = Math.max(...nodes.map(node => node.column));
        const graphWidth = (maxColumn + 1) * this.options.columnWidth;
        
        // Group commits by date
        const dateGroups = new Map<string, { date: Date, minY: number, maxY: number, nodes: GraphNode[] }>();
        
        nodes.forEach(node => {
            const pos = this.positionCache.get(node.id);
            if (!pos) return;
            
            const date = new Date(node.timestamp * 1000);
            const dateKey = date.toLocaleDateString();
            
            if (!dateGroups.has(dateKey)) {
                dateGroups.set(dateKey, { 
                    date, 
                    minY: pos.y, 
                    maxY: pos.y,
                    nodes: [node]
                });
            } else {
                const group = dateGroups.get(dateKey)!;
                group.minY = Math.min(group.minY, pos.y);
                group.maxY = Math.max(group.maxY, pos.y);
                group.nodes.push(node);
            }
        });
        
        // Sort date groups by timestamp (newest first)
        const sortedGroups = Array.from(dateGroups.values())
            .sort((a, b) => b.date.getTime() - a.date.getTime());
        
        // Draw date markers
        sortedGroups.forEach(group => {
            // Only draw markers for dates with multiple commits or if there are few dates
            if (group.nodes.length > 1 || sortedGroups.length <= 10) {
                const centerY = (group.minY + group.maxY) / 2;
                
                // Draw horizontal line
                const line = document.createElementNS(this.svgns, "line");
                line.setAttribute("x1", "0");
                line.setAttribute("y1", centerY.toString());
                line.setAttribute("x2", graphWidth.toString());
                line.setAttribute("y2", centerY.toString());
                line.setAttribute("class", "revision-graph__date-marker");
                line.setAttribute("stroke-dasharray", "2,2");
                line.setAttribute("stroke", "rgba(128, 128, 128, 0.3)");
                line.setAttribute("stroke-width", "1");
                
                // Add date label
                const label = document.createElementNS(this.svgns, "text") as SVGTextElement;
                label.setAttribute("x", (graphWidth + 10).toString());
                label.setAttribute("y", (centerY + 4).toString());
                label.setAttribute("class", "revision-graph__date-label");
                label.setAttribute("fill", "rgba(128, 128, 128, 0.7)");
                label.setAttribute("font-size", "12");
                label.textContent = this.formatDateForMarker(group.date);
                
                // Add background for better readability
                // Cast to SVGTextElement to access getBBox method
                const textWidth = group.date.toLocaleDateString().length * 7;
                const textHeight = 14;
                
                // Try to get actual dimensions if possible
                let bbox = { width: textWidth, height: textHeight };
                try {
                    // Temporarily add to DOM to get dimensions
                    this.svg.appendChild(label);
                    const svgBBox = label.getBBox();
                    if (svgBBox) {
                        bbox = svgBBox;
                    }
                    this.svg.removeChild(label);
                } catch (e) {
                    // Fallback to estimated dimensions if getBBox fails
                    console.warn("Could not get text dimensions, using estimates", e);
                }
                
                const padding = 4;
                const rect = document.createElementNS(this.svgns, "rect");
                rect.setAttribute("x", (graphWidth + 8).toString());
                rect.setAttribute("y", (centerY - 8).toString());
                rect.setAttribute("width", (bbox.width + padding * 2).toString());
                rect.setAttribute("height", (bbox.height + padding).toString());
                rect.setAttribute("fill", "rgba(255, 255, 255, 0.8)");
                rect.setAttribute("rx", "2");
                rect.setAttribute("class", "revision-graph__date-label-bg");
                
                // Add elements to SVG
                this.svg.appendChild(line);
                this.svg.appendChild(rect);
                this.svg.appendChild(label);
            }
        });
    }
    
    /**
     * Formats a date for display in the date marker
     * @param date - Date to format
     * @returns Formatted date string
     */
    private formatDateForMarker(date: Date): string {
        const now = new Date();
        const isCurrentYear = date.getFullYear() === now.getFullYear();
        
        // Use different format based on how recent the date is
        if (isCurrentYear) {
            // For current year, show month and day
            return date.toLocaleDateString(undefined, { month: 'short', day: 'numeric' });
        } else {
            // For other years, include the year
            return date.toLocaleDateString(undefined, { month: 'short', day: 'numeric', year: 'numeric' });
        }
    }
}
