import type { Commit, GraphOptions, ActionLink, UITextOptions } from "./types";

/**
 * Options for the revision list generator
 */
export interface RevisionListOptions {
    /** Container element to render the list in */
    container: HTMLElement;
    /** Theme for the list (light/dark) */
    theme?: "light" | "dark";
    /** Callback when a commit is selected */
    onCommitSelect?: (commitId: string) => void;
    /** Callback when an action is clicked */
    onActionClick?: (url: string, commitId: string) => void;
    /** Additional options for customization */
    colors?: string[];
    width?: number;
    nodeHeight?: number;
    nodeWidth?: number;
    columnWidth?: number;
    showLabels?: boolean;
    branchColors: Map<string, string>;
    uiText?: UITextOptions;
}

/**
 * Default UI text options
 */
const DEFAULT_UI_TEXT: UITextOptions = {
    revisionTerm: "Commit",
    historyTerm: "Git commit history",
    graphVisualizationLabel: "Git commit graph visualization",
    interactiveGraphLabel: "Interactive commit graph",
    graphDescription: "Visual representation of Git commit history showing branches and connections between commits",
    detailsLabel: "Commit details",
    closeButtonLabel: "Close commit details",
    actionsLabel: "Commit actions",
    historyListLabel: "Revision history"
};

/**
 * RevisionListGenerator class for creating a modern, accessible list view of Git commits
 * @example
 * ```typescript
 * const generator = new RevisionListGenerator(container, { theme: "dark" });
 * generator.generate(commits);
 * ```
 */
export class RevisionListGenerator {
    private readonly container: HTMLElement;
    private readonly options: RevisionListOptions;
    private readonly uiText: UITextOptions;
    private readonly listContainer: HTMLElement;
    private selectedCommitId: string | null = null;
    private readonly eventListeners: Map<string, (e: Event) => void> = new Map();

    /**
     * Creates a new instance of RevisionListGenerator
     * @param container - The container element to render the list in
     * @param options - Optional configuration options for the list
     */
    constructor(container: HTMLElement, options: Partial<RevisionListOptions> = {}) {
        this.container = container;
        
        // Set default options
        this.options = {
            container,
            theme: options.theme,
            onCommitSelect: options.onCommitSelect,
            onActionClick: options.onActionClick,
            colors: options.colors,
            width: options.width,
            nodeHeight: options.nodeHeight,
            nodeWidth: options.nodeWidth,
            columnWidth: options.columnWidth,
            showLabels: options.showLabels,
            branchColors: new Map(),
            ...options
        };

        // Set UI text options
        this.uiText = { ...DEFAULT_UI_TEXT, ...options.uiText };
        
        // Create and configure list container
        this.listContainer = document.createElement("div");
        this.listContainer.className = "revision-graph__list";
        this.listContainer.setAttribute("role", "list");
        this.listContainer.setAttribute("aria-label", `${this.uiText.historyListLabel || DEFAULT_UI_TEXT.historyListLabel}`);
        
        // Set theme if provided
        if (options.theme === "dark") {
            this.container.setAttribute("data-theme", "dark");
        }

        // Add list container to main container
        this.container.appendChild(this.listContainer);
    }

    /**
     * Generates the commit list (alias for render)
     * @param commits - Array of commits to render
     */
    public generate(commits: Commit[]): void {
        this.render(commits);
    }

    /**
     * Renders the commit list
     * @param commits - Array of commits to render
     */
    public render(commits: Commit[]): void {
        // Clear existing content
        this.listContainer.innerHTML = "";

        // Sort commits by timestamp (newest first)
        const sortedCommits = [...commits].sort((a, b) => b.timestamp - a.timestamp);

        // Create and append commit items
        sortedCommits.forEach((commit, index) => {
            const item = this.createCommitListItem(commit, index);
            this.listContainer.appendChild(item);
        });
    }

    /**
     * Creates a commit list item
     */
    private createCommitListItem(commit: Commit, index: number): HTMLDivElement {
        const item = document.createElement("div");
        item.className = "revision-graph__item";
        item.setAttribute("role", "listitem");
        item.setAttribute("aria-label", `${this.uiText.revisionTerm || DEFAULT_UI_TEXT.revisionTerm} ${commit.id.substring(0, 7)} by ${commit.author}`);
        item.setAttribute("tabindex", "0");
        item.setAttribute("data-commit-id", commit.id);
        item.setAttribute("data-index", index.toString());

        // Add event listeners
        item.addEventListener("click", () => this.handleItemClick(commit, index));
        item.addEventListener("keydown", (e) => {
            if (e.key === "Enter" || e.key === " ") {
                e.preventDefault();
                this.handleItemClick(commit, index);
            }
        });

        // Create content container
        const content = document.createElement("div");
        content.className = "revision-graph__content";

        // Add branch label if present
        if (commit.refs && commit.refs.length > 0) {
            const branchLabel = document.createElement("div");
            branchLabel.className = "revision-graph__branch-label";
            branchLabel.textContent = commit.refs[0];
            branchLabel.style.backgroundColor = this.getBranchColor(commit.refs[0]);
            content.appendChild(branchLabel);
        }

        // Add actions dropdown
        const actionsDropdown = document.createElement("div");
        actionsDropdown.className = "revision-graph__actions revision-graph__actions--top-right";
        this.createActionsDropdown(actionsDropdown, commit);
        content.appendChild(actionsDropdown);

        // Create header
        const header = document.createElement("div");
        header.className = "revision-graph__header";

        // Create message container
        const messageContainer = document.createElement("div");
        messageContainer.className = "revision-graph__message-container";

        // Create header content
        const headerContent = document.createElement("div");
        headerContent.className = "revision-graph__header-content";

        // Add title
        const title = document.createElement("div");
        title.className = "revision-graph__title";
        title.innerHTML = commit.message;
        headerContent.appendChild(title);

        // Add hash
        const hash = document.createElement("code");
        hash.className = "revision-graph__hash";
        hash.textContent = commit.id.substring(0, 7);
        headerContent.appendChild(hash);

        messageContainer.appendChild(headerContent);

        header.appendChild(messageContainer);
        content.appendChild(header);

        // Add metadata
        const meta = document.createElement("div");
        meta.className = "revision-graph__meta";

        // Add author
        const author = document.createElement("span");
        author.className = "revision-graph__author";
        author.textContent = commit.author;
        meta.appendChild(author);

        // Add date
        const date = document.createElement("time");
        date.className = "revision-graph__date";
        const commitDate = new Date(commit.timestamp * 1000);
        date.dateTime = commitDate.toISOString();
        date.title = commitDate.toLocaleString();
        date.textContent = this.formatRelativeTime(commitDate);
        meta.appendChild(date);

        content.appendChild(meta);

        // Add tags if present
        if (commit.tags && commit.tags.length > 0) {
            const labels = document.createElement("div");
            labels.className = "revision-graph__labels";

            commit.tags.forEach(tag => {
                const tagElement = document.createElement("span");
                tagElement.className = "revision-graph__tag";
                tagElement.textContent = tag.label;

                if (tag.color) {
                    tagElement.style.backgroundColor = `${tag.color}15`;
                    tagElement.style.color = tag.color;
                    tagElement.style.borderColor = tag.color;
                }

                labels.appendChild(tagElement);
            });

            content.appendChild(labels);
        }
        item.appendChild(content);
        return item;
    }

    /**
     * Creates the actions dropdown for a commit
     */
    private createActionsDropdown(container: HTMLDivElement, commit: Commit): void {
        // Create button
        const button = document.createElement("button");
        button.className = "revision-graph__actions-btn";
        button.setAttribute("aria-label", `${this.uiText.actionsLabel || DEFAULT_UI_TEXT.actionsLabel}`);
        button.setAttribute("aria-haspopup", "true");
        button.setAttribute("aria-expanded", "false");
        button.innerHTML = "⋮";

        // Create dropdown menu
        const menu = document.createElement("div");
        menu.className = "revision-graph__dropdown";
        menu.setAttribute("role", "menu");
        menu.setAttribute("aria-label", `${this.uiText.actionsLabel || DEFAULT_UI_TEXT.actionsLabel}`);

        // Add URLs as menu items
        if (commit.urls && commit.urls.length > 0) {
            commit.urls.forEach((url, index) => {
                if (index > 0) {
                    const divider = document.createElement("div");
                    divider.className = "revision-graph__dropdown-divider";
                    menu.appendChild(divider);
                }

                const item = document.createElement("a");
                item.className = "revision-graph__dropdown-item";
                item.setAttribute("role", "menuitem");
                item.href = url.url;
                item.textContent = url.label || url.url;
                item.target = "_blank";
                item.rel = "noopener noreferrer";

                menu.appendChild(item);
            });
        }

        // Add event listeners
        button.addEventListener("click", (e) => {
            e.stopPropagation();
            this.toggleDropdown(menu, button);
        });

        // Add to container
        container.appendChild(button);
        container.appendChild(menu);
    }

    /**
     * Toggles the dropdown menu
     */
    private toggleDropdown(menu: HTMLDivElement, button: HTMLButtonElement): void {
        const isOpen = menu.style.display === "block";

        // Close all other dropdowns
        document.querySelectorAll(".revision-graph__dropdown").forEach(dropdown => {
            if (dropdown !== menu) {
                (dropdown as HTMLElement).style.display = "none";
                const btn = dropdown.previousElementSibling as HTMLButtonElement;
                if (btn) {
                    btn.setAttribute("aria-expanded", "false");
                }
            }
        });

        // Toggle this dropdown
        if (isOpen) {
            menu.style.display = "none";
            button.setAttribute("aria-expanded", "false");
        } else {
            menu.style.display = "block";
            button.setAttribute("aria-expanded", "true");

            // Add document click listener to close dropdown
            setTimeout(() => {
                const closeHandler = (e: MouseEvent) => {
                    if (!menu.contains(e.target as Node) && e.target !== button) {
                        menu.style.display = "none";
                        button.setAttribute("aria-expanded", "false");
                        document.removeEventListener("click", closeHandler);
                    }
                };
                document.addEventListener("click", closeHandler);
            }, 0);
        }
    }

    /**
     * Handles click on a list item
     */
    private handleItemClick(commit: Commit, index: number): void {
        // Update selected item
        this.setSelectedItem(index, commit.id);

        // Trigger callback if provided
        if (this.options.onCommitSelect) {
            this.options.onCommitSelect(commit.id);
        }
    }

    /**
     * Sets the selected item
     */
    private setSelectedItem(index: number, commitId: string): void {
        // Remove selection from previous item
        const previousItem = this.listContainer.querySelector(".revision-graph__item--selected");
        previousItem?.classList.remove("revision-graph__item--selected");

        // Add selection to new item
        const newItem = this.listContainer.querySelector(`[data-index="${index}"]`);
        newItem?.classList.add("revision-graph__item--selected");

        // Store selected index
        this.selectedCommitId = commitId;
    }

    /**
     * Formats a date into a relative time string
     * @param date - Date to format
     * @returns Formatted string (e.g., "2h ago")
     */
    private formatRelativeTime(date: Date): string {
        const now = new Date();
        const diffInSeconds = Math.floor((now.getTime() - date.getTime()) / 1000);
        const diffInMinutes = Math.floor(diffInSeconds / 60);
        const diffInHours = Math.floor(diffInMinutes / 60);
        const diffInDays = Math.floor(diffInHours / 24);

        if (diffInSeconds < 60) {
            return "just now";
        } else if (diffInMinutes < 60) {
            return `${diffInMinutes}m ago`;
        } else if (diffInHours < 24) {
            return `${diffInHours}h ago`;
        } else if (diffInDays < 30) {
            return `${diffInDays}d ago`;
        } else {
            return date.toLocaleDateString();
        }
    }

    /**
     * Handles action link clicks
     * @param url - URL of the action
     * @param commitId - ID of the associated commit
     */
    private handleActionClick(url: string, commitId: string): void {
        this.options.onActionClick?.(url, commitId);
    }

    /**
     * Cleans up resources and event listeners
     */
    public cleanup(): void {
        this.eventListeners.forEach((handler, event) => {
            this.container.removeEventListener(event, handler);
        });
        this.eventListeners.clear();
        this.container.innerHTML = "";
    }

    /**
     * Gets the color for a branch
     * @param branch - Branch name
     * @returns Color string
     */
    private getBranchColor(branch: string): string {
        const colors = this.options.colors || ["#222222"];
        const index = Math.abs(this.hashString(branch)) % colors.length;
        return colors[index];
    }

    /**
     * Creates a hash code from a string
     * @param str - String to hash
     * @returns Hash code
     */
    private hashString(str: string): number {
        let hash = 0;
        for (let i = 0; i < str.length; i++) {
            const char = str.charCodeAt(i);
            hash = ((hash << 5) - hash) + char;
            hash = hash & hash;
        }
        return hash;
    }
}
