/**
 * Represents a single commit in the Git history
 */
export interface Commit {
    /** Unique identifier for the commit (hash) */
    id: string;
    /** Array of parent commit IDs */
    parents: string[];
    /** Commit message */
    message: string;
    /** Author of the commit */
    author: string;
    /** Timestamp of when the commit was created */
    timestamp: number;
    /** Branch name or tag associated with this commit */
    refs?: string[];
    /** URLs associated with the commit */
    urls?: Array<{
        label: string;
        url: string;
    }> | null;
    /** Tags associated with the commit */
    tags?: Array<{
        name: string;
        color: string;
        label: string;
    }> | null;
}

/**
 * UI text customization options
 */
export interface UITextOptions {
    /** Term used for a single content state (default: "Commit") */
    revisionTerm?: string;
    /** Term used for content history (default: "Git commit history") */
    historyTerm?: string;
    /** Text for the revision graph visualization label (default: "Git commit graph visualization") */
    graphVisualizationLabel?: string;
    /** Text for the interactive graph label (default: "Interactive commit graph") */
    interactiveGraphLabel?: string;
    /** Text for the graph description (default: "Visual representation of Git commit history...") */
    graphDescription?: string;
    /** Text for revision details label (default: "Commit details") */
    detailsLabel?: string;
    /** Text for close button (default: "Close commit details") */
    closeButtonLabel?: string;
    /** Text for actions dropdown (default: "Commit actions") */
    actionsLabel?: string;
    /** Text for revision history list (default: "Revision history") */
    historyListLabel?: string;
}

/**
 * Configuration options for the graph generator
 */
export interface GraphOptions {
    /** Width of the graph in pixels */
    width: number;
    /** Height of each commit node in pixels */
    nodeHeight: number;
    /** Width of each commit node in pixels */
    nodeWidth: number;
    /** Horizontal spacing between branches */
    columnWidth: number;
    /** Colors to use for different branches */
    colors: string[];
    /** Whether to show labels */
    showLabels: boolean;
    /** Theme configuration */
    theme: "light" | "dark";
    /** Colors to use for different branches */
    branchColors: Map<string, string>;
    /** UI text customization options */
    uiText?: UITextOptions;
}

/**
 * Represents a node in the processed graph
 */
export interface GraphNode {
    /** The commit this node represents */
    id: string;
    /** The commit message */
    message: string;
    /** The author of the commit */
    author: string;
    /** The timestamp of when the commit was created */
    timestamp: number;
    /** Array of parent commit IDs */
    parents: string[];
    /** Branch name or tag associated with this commit */
    refs: string[];
    /** Tags associated with the commit */
    tags: Array<{ name: string; color: string; label: string; }>;
    /** URLs associated with the commit */
    urls: ActionLink[];
    /** Column index in the graph */
    column: number;
    /** Connections to parent nodes */
    connections: Connection[];
    /** Color of the node */
    color: string;
}

/**
 * Represents a connection between two nodes
 */
export interface Connection {
    /** Starting point coordinates */
    from: GraphNode;
    /** Ending point coordinates */
    to: GraphNode;
    /** Color of the connection line */
    color: string;
    /** Optional type property to specify connection style */
    type?: "direct" | "bridge";
}

/**
 * Represents a point in the graph
 */
export interface Point {
    x: number;
    y: number;
}

export interface ActionLink {
    label: string;
    url: string;
}

const DEFAULT_OPTIONS: GraphOptions = {
    width: 800,
    nodeHeight: 50,
    nodeWidth: 100,
    columnWidth: 200,
    colors: ["#e11d21", "#fbca04", "#009800", "#006b75", "#207de5", "#0052cc", "#5319e7"],
    showLabels: true,
    theme: "light",
    branchColors: new Map<string, string>(),
}; 