<?php

namespace Drupal\revision_graph\Controller;

use Drupal\Component\Utility\Xss;
use Drupal\Core\Database\Connection;
use Drupal\Core\Datetime\DateFormatterInterface;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Entity\EntityRepositoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\Render\RendererInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Url;
use Drupal\node\NodeInterface;
use Drupal\node\NodeStorageInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;

/**
 * Controller for the Revision Graph functionality.
 * 
 * Provides endpoints and helper methods for retrieving and formatting
 * revision data to power the graphical revision interface.
 */
class RevisionGraphController implements ContainerInjectionInterface {

  /**
   * The maximum number of revisions to load per page/request.
   *
   * @var int
   */
  const PAGE_LIMIT = 50;

  use StringTranslationTrait;

  /**
   * {@inheritdoc}
   */
  public function __construct(
    private readonly DateFormatterInterface $date_formatter,
    private readonly RendererInterface $renderer,
    private readonly EntityRepositoryInterface $entityRepository,
    private readonly Connection $connection,
    private readonly EntityTypeManagerInterface $entityTypeManager,
    private readonly LanguageManagerInterface $languageManager) {}

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('date.formatter'),
      $container->get('renderer'),
      $container->get('entity.repository'),
      $container->get('database'),
      $container->get('entity_type.manager'),
      $container->get('language_manager')
    );
  }

  /**
   * Renders the revision graph overview page.
   *
   * @param \Drupal\node\NodeInterface $node
   *   The node entity to display revisions for.
   *
   * @return array
   *   A render array for the revision graph page.
   */
  public function revisionOverview(NodeInterface $node) {
    return [
      '#theme' => 'revision_graph',
      '#total' => $this->getTotalRevisions($node),
      '#limit' => self::PAGE_LIMIT,
      '#attached' => [
        'library' => 'revision_graph/revision_graph',
      ],
    ];
  }

  /**
   * Loads revision data for all languages of a node.
   *
   * @param \Drupal\node\NodeInterface $node
   *   The node entity to load revisions for.
   * @param int $offset
   *   The offset for pagination.
   * @param int $limit
   *   The number of revisions to load per page.
   *
   * @return array
   *   An array of revision data formatted as commit objects.
   */
  protected function loadRevisions(NodeInterface $node, $offset = 0, $limit = self::PAGE_LIMIT) {
    $commits = [];
    // Irrespective of which version/language of the node is provided, we
    // use the active node for calculation.
    $node = $this->entityRepository->getActive($node->getEntityTypeId(), $node->id());
    if ($node instanceof NodeInterface) {
      foreach ($this->languageManager->getLanguages() as $language) {
        $revisions = $this->revisionOverviewItems($node, $language->getId(), $offset, $limit);
        foreach ($this->revisionOverviewAsFakeCommits($node, $language->getId(), $revisions) as $commit) {
          $commits[] = $commit;
        }
      }
    }
    return $commits;
  }

  /**
   * JSON endpoint to retrieve revision data for AJAX loading.
   *
   * @param \Drupal\node\NodeInterface $node
   *   The node entity to load revisions for.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   A JSON response containing the revision data.
   */
  public function revisions(NodeInterface $node): JsonResponse {
    $offset = \Drupal::request()->query->get('offset');
    $limit = \Drupal::request()->query->get('limit');
    $commits = $this->loadRevisions($node, (int) $offset, (int) $limit);
    return new JsonResponse($commits);
  }

  /**
   * Generates revision overview items for a specific language.
   *
   * @param \Drupal\node\NodeInterface $node
   *   The node entity to load revisions for.
   * @param string|null $language_code
   *   The language code to filter revisions by.
   * @param int $offset
   *   The offset for pagination.
   * @param int $limit
   *   The number of revisions to load per page.
   *
   * @return array
   *   An array of revision data.
   */
  public function revisionOverviewItems(NodeInterface $node, $language_code = NULL, $offset = 0, $limit = self::PAGE_LIMIT) {
    // Always use the latest revision in the current content language to
    // determine if this node has translations. This supports showing the
    // correct translation revisions for translations that only have.
    // non-default revisions.
    $node = $this->entityRepository->getActive($node->getEntityTypeId(), $node->id());
    $langcode = $language_code ?: $node->language()->getId();
    $languages = $node->getTranslationLanguages();
    $has_translations = (count($languages) > 1);
    $node_storage = $this->entityTypeManager->getStorage('node');
    $rows = [];
    $current_revision_displayed = FALSE;
    $revision_ids = $this->getRevisionIds($node, $node_storage, $offset, $limit);
    foreach ($revision_ids as $vid) {
      /** @var \Drupal\node\NodeInterface $revision */
      $revision = $node_storage->loadRevision($vid);
      // Only show revisions that are affected by the language that is being
      // displayed.
      if ($revision->hasTranslation($langcode) && $revision->getTranslation($langcode)->isRevisionTranslationAffected()) {
        // We treat also the latest translation-affecting revision as current
        // revision, if it was the default revision, as its values for the
        // current language will be the same of the current default revision in
        // this case.
        $is_current_revision = $revision->isDefaultRevision() || (!$current_revision_displayed && $revision->wasDefaultRevision());
        if ($is_current_revision) {
          $current_revision_displayed = TRUE;
        }

        $message = ['#markup' => $revision->revision_log->value, '#allowed_tags' => Xss::getHtmlTagList()];
        $row = [
          'is_current_revision' => $is_current_revision,
          'timestamp' => $revision->revision_timestamp->value,
          'author' => $revision->getRevisionUser()->label(),
          'message' => (string) $this->renderer->renderInIsolation($message),
        ];
        $links = [];
        $view_url = new Url('entity.node.revision', ['node' => $node->id(), 'node_revision' => $vid]);
        $links['view'] = [
          'label' => (string) $this->t('View'),
          'url' => $view_url->toString(),
        ];
        if (!$is_current_revision) {
          if ($revision->access('revert revision')) {

            if ($has_translations) {
              $revert_url = Url::fromRoute('node.revision_revert_translation_confirm', ['node' => $node->id(), 'node_revision' => $vid, 'langcode' => $langcode]);
            }
            else {
              $revert_url = Url::fromRoute('node.revision_revert_confirm', ['node' => $node->id(), 'node_revision' => $vid]);
            }
            $links['revert'] = [
              'label' => $vid < $node->getRevisionId() ? $this->t('Revert') : $this->t('Set as current revision'),
              'url' => $revert_url->toString(),
            ];
          }
          if ($revision->access('delete revision')) {
            $delete_url = Url::fromRoute('node.revision_delete_confirm', ['node' => $node->id(), 'node_revision' => $vid]);
            $links['delete'] = [
              'label' => $this->t('Delete'),
              'url' => $delete_url->toString(),
            ];
          }
        }
        $row['urls'] = array_values($links);
        $rows[$vid] = $row;
      }
    }
    return $rows;
  }


  /**
   * Gets the total number of revisions for a node.
   *
   * @param \Drupal\node\NodeInterface $node
   *   The node entity.
   *
   * @return int
   *   The total number of revisions.
   */
  protected function getTotalRevisions(NodeInterface $node) {
    $query = $this->entityTypeManager->getStorage('node')->getQuery()
      ->accessCheck(TRUE)
      ->allRevisions()
      ->condition($node->getEntityType()->getKey('id'), $node->id())
      ->count();
    return $query->execute();
  }

  /**
   * Gets a list of node revision IDs for a specific node.
   *
   * @param \Drupal\node\NodeInterface $node
   *   The node entity.
   * @param \Drupal\node\NodeStorageInterface $node_storage
   *   The node storage handler.
   * @param int $offset
   *   The offset for pagination.
   * @param int|null $limit
   *   The number of revisions to load per page, or NULL to load all revisions.
   *
   * @return int[]
   *   Node revision IDs (in descending order).
   */
  protected function getRevisionIds(NodeInterface $node, NodeStorageInterface $node_storage, $offset = 0, $limit = NULL) {
    $query = $node_storage->getQuery()
      ->accessCheck(TRUE)
      ->allRevisions()
      ->condition($node->getEntityType()->getKey('id'), $node->id())
      ->sort($node->getEntityType()->getKey('revision'), 'DESC');
    if ($limit) {
      $query->range($offset, $limit);
    }
    $result = $query->execute();
    return array_keys($result);
  }

  /**
   * Formats revision data as "commits" for graph visualization.
   *
   * This method transforms standard Drupal revision data into a format
   * compatible with git-like graph visualization, including branch information
   * and revision tags.
   *
   * @param \Drupal\node\NodeInterface $node
   *   The node entity.
   * @param string $langcode
   *   The language code to filter revisions by.
   * @param array $revisions
   *   An array of revision data from revisionOverviewItems().
   *
   * @return array
   *   An array of "commit" objects compatible with the graph visualization.
   */
  protected function revisionOverviewAsFakeCommits(NodeInterface $node, $langcode, $revisions = []) {
    $tags_list = [
      'published' => [
        'name' => 'published',
        'label' => $this->t('Published'),
        'color' => '#4d6aff',
      ],
      'current' => [
        'name' => 'current',
        'label' => $this->t('Current'),
        'color' => '#ff6d1f',
      ],
      'live' => [
        'name' => 'live',
        'label' => $this->t('Live'),
        'color' => '#28a745',
      ],
      'latest' => [
        'name' => 'latest',
        'label' => $this->t('Latest revision'),
        'color' => '#681b98',
      ],
      'default' => [
        'name' => 'default',
        'label' => $this->t('Default'),
        'color' => '#0D142B',
      ],
    ];
    $commits = [];
    $node_storage = $this->entityTypeManager->getStorage('node');
    foreach ($revisions as $vid => $revision_data) {
      $commit = [
        'id' => (string) $vid,
        'author' => $revision_data['author'],
        'timestamp' => $revision_data['timestamp'],
        'urls' => $revision_data['urls'],
        'message' => $revision_data['message'],
        'branch' => $langcode,
        'parents' => [],
        'tags' => [],
      ];
      /** @var \Drupal\node\NodeInterface $revision */
      $revision = $node_storage->loadRevision($vid);
      $revision = $revision->getTranslation($langcode);
      // Subject can be a good candidate for moderation state.
      if ($revision_data['is_current_revision']) {
        $commit['tags'][] = $tags_list['current'];
      }
      if ($revision->isLatestRevision()) {
        $commit['tags'][] = $tags_list['latest'];
      }
      if ($revision->isPublished()) {
        $commit['tags'][] = $tags_list['published'];
        if ($revision_data['is_current_revision']) {
          $commit['tags'][] = $tags_list['live'];
        }
      }
      if ($revision->isDefaultRevision()) {
        $commit['tags'][] = $tags_list['default'];
      }
      $commits[(int) $vid] = $commit;
    }
    return $commits;
  }

}
